# C# 关键字和标识符

> 原文： [https://www.programiz.com/csharp-programming/keywords-identifiers](https://www.programiz.com/csharp-programming/keywords-identifiers)

#### 在本教程中，我们将学习 C# 编程语言中的关键字（保留字）和标识符。

## C# 关键字

关键字是预定义的保留字集，在程序中具有特殊含义。 关键字的含义不能更改，也不能直接用作程序中的标识符。

例如，

```cs
long mobileNum;
```

在此，`long`是关键字， `mobileNum` 是变量（标识符）。 `long`在 C# 中具有特殊含义，即，它用于声明`long`类型的变量，并且该函数无法更改。

另外，不能将`long`，`int`，`char`等关键字用作标识符。 因此，我们不能有以下内容：

```cs
long long;
```

C# 总共有 79 个关键字。 所有这些关键字都是小写的。 这是所有 C# 关键字的完整列表。

| 摘要 | 如 | 基础 | 布尔 |
| 打破 | 字节 | 案件 | 抓住 |
| 烧焦 | 已检查 | 类 | const |
| 继续 | 小数 | 默认 | 代表 |
| 做 | 双 | 其他 | 枚举 |
| 事件 | 显性的 | 外部 | 假 |
| 最后 | 固定 | 浮动 | 对于 |
| 前言 | 去 | 如果 | 隐含的 |
| 在 | 在（通用修饰符） | 整型 | 接口 |
| 内部 | 是 | 锁 | long |
| 命名空间 | 新 | 空值 | 目的 |
| 算子 | 出 | out（通用修饰符） | 覆写 |
| 参数 | 私人的 | 受保护的 | 上市 |
| 只读 | 参考 | 返回 | 兆字节 |
| 密封 | 短 | 大小 | 栈分配 |
| 静态的 | 串 | 结构 | 开关 |
| 这个 | 扔 | 真正 | 尝试 |
| 类型 | int | 乌龙 | 未检查 |
| 不安全的 | 超短 | 使用 | 使用静态 |
| 虚空 | 易挥发的 | 而 |   |

尽管关键字是保留字，但是如果将`@`添加为前缀，它们可以用作标识符。 例如，

```cs
int @void;
```

上面的语句将创建类型为`int`的变量 `@void` 。

* * *

### 内容比对关键字

除了常规关键字，C# 还具有 25 个上下文关键字。 上下文关键字在有限的程序上下文中具有特定含义，并且可以用作该上下文之外的标识符。 它们不是 C# 中的保留字。

| 加 | 别名 | 上升 |
| 异步的 | 等待 | 降序 |
| 动态 | 从 | 得到 |
| 全球 | 组 | 进入 |
| 加入 | 让 | 订购 |
| 部分（类型） | 部分（方法） | 去掉 |
| 选择 | 组 | 值 |
| 哪里 | 何时（过滤条件） | 哪里（通用类型约束） |
| 让 |   |   |

如果您想了解每个关键字的功能，建议您访问 [C# 关键字](https://docs.microsoft.com/en-us/dotnet/csharp/language-reference/keywords/index)（官方 C# 文档）。

* * *

## C# 标识符

标识符是给变量，方法，类等实体的名称。它们是程序中唯一标识元素的令牌。 例如，

```cs
int value;
```

在此，`value`是变量的名称。 因此，它是一个标识符。 除非添加`@`作为前缀，否则保留的关键字不能用作标识符。 例如，

```cs
int break;
```

该语句将在编译时生成错误。

要了解有关变量的更多信息，请访问 [C# 变量](/csharp-programming/variables-primitive-data-types "C# variables")。

* * *

## 命名标识符的规则

*   标识符不能是 C# 关键字。
*   标识符必须以字母，下划线或`@`符号开头。 标识符的其余部分可以包含字母，数字和下划线符号。
*   不允许使用空格。 除字母，数字和下划线外，它都不能有其他符号。
*   标识符区分大小写。 因此， `getName` ， `GetName` 和 `getname` 代表 3 个不同的标识符。

以下是一些有效和无效的标识符：

| 身份标识 | 备注 |
| --- | --- |
| 数 | 有效 |
| 计算标记 | Valid |
| 你好$ | 无效（包含$） |
| 名称 1 | Valid |
| @如果 | 有效（关键字前缀为@） |
| if | 无效（C# 关键字） |
| 我的名字 | 无效（包含空格） |
| _ 你好 | Valid |

* * *

## 示例：在程序中查找关键字和标识符的列表

为了澄清概念，让我们在 [C# Hello World](/csharp-programming/hello-world "C# Hello World Program") 中编写的程序中找到关键字和标识符的列表。

```cs
using System;
namespace HelloWorld
{
    class Hello
{         
        static void Main(string[] args)
        {
          Console.WriteLine("Hello World!");
        }
    }
}
```

| 关键词 | Identifiers |
| --- | --- |
| using | 系统 |
| namespace | HelloWorld（命名空间） |
| class | 同学们好） |
| static | 主（方法） |
| void | args |
| string | 安慰 |
|   | 写行 |

<samp>“ Hello World！” `WriteLine`方法内的</samp>是字符串文字。